<?php
/******************************* PHP-P10 v2 *****************************
 * Copyright (C) 2011  Philipp Kreil (pk910)                            *
 *                                                                      *
 * This program is free software: you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation, either version 3 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      * 
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program. If not, see <http://www.gnu.org/licenses/>. *
 *                                                                      *
 ************************************************************************
 * 
 *  Bots/ExampleBot.class.php
 *
 * a simple example bot...
 *
 */

class {$_NAME} extends Bot {  // {$_NAME} will be replaced by our script later ;)
	private $uplink;
	private $example_bot;
 
	public function load($uplink, $old = false) { //load is called when the module gets included
		$this->uplink = $uplink; //Reference to the P10 Uplink
		
		//$old is only set, if the module is rehashed. It contains the return value of unload();
		
		if(!$old) {
			
			//Here you can define startup actions
			//We'll add an user to the Server
			// newuser($localid,$nick,$ident,$host,$realname,$uptime,$modes)   returns an array with the result
			// $localid  is just an id you can set (whatever you want) - maybe you'll use it to identify the bot/user later?
			
			$nick = "ExampleBot"; // Please note: If this user already exists on another Server it will be killed!
			$ident = "Example";
			$host = "Example.Bot";
			$realname = "Thats an example Bot :)";
			$modes = "i"; //we don't need a leading + (that will be added automatically)
			
			$this->example_bot = $this->uplink->addUser($nick,$ident,$host,$realname,$uptime,$modes); //addUser($nick, $ident, $host, $ip, $modes, $realname)
			if(is_a($this->example_bot, "P10_User")) { // A new user was created :)
				//ok  let's join a channel
				$this->uplink->join($this->example_bot, "#test");
				//now we want to say something...
				//but note: thats the startup procedure! the p10 server is not connected to an uplink, yet - so noone would recive our message (only the other bots on this server)
			}
			
		} else {
			$this->example_bot = $old; //We've saved out Bot reference in $old so we can simply use it again...
		}
		
		//OK  send something to the Server is easy...
		//How to recive something from the IRC Server?
		
		//I've copied the eggdrop way to do that - bind
		ModCMD::bind($this, BIND_PRIVMSG, "recive_privmsg");
		ModCMD::bind($this, BIND_QUIT, "recive_quit");
	}
	
	public function loop() { //this function is triggered as often as possible (at least one time per second!). Maybe you need it to do timed events
		//please don't trigger any blocking functions here... that would cause an extreme lagg!
	}
	
	public function unload($rehash = false) { //this function is triggered, when the Bot is unloaded... If it's just a rehash the return value of this method is passed to $old in the load method.
		if($rehash) {
			return $this->example_bot;
		} else {
			$this->uplink->delUser($this->example_bot, "Bye.");
		}
	}
	
	public function recive_privmsg($user, $channel, $message) {
		//We've got a privmsg...
		$exp=explode(" ",$message);
		if($exp[0] == "hi") {
			$this->uplink->privmsg($this->example_bot, $channel, "Hallo ".$user->getNick()); // We send a message back - documentation is following
		}
		if($exp[0] == "-users" && is_a($channel, "P10_Channel")) {
			$user_str = "";
			foreach($channel->getUsers() as $chan_user) {
				$extra="";
				$privs = $channel->getUserPrivs($chan_user);
				if(($privs & P10_Channel::USERPRIV_VOICE)) $extra= '+'; 
				if(($privs & P10_Channel::USERPRIV_OPED)) $extra= '@'; 
				$user_str .= " " . $extra . $chan_user->getNick();
			}
			$this->uplink->notice($this->example_bot, $user, "User in ".$channel->getName().":".$user_str);
		}
	}
	
	function recive_quit($user, $reason) {
		if($user === $this->example_bot) { //maybe we got killed???
			$this->load($this->uplink);
		}
	}
	
}

?>